{
Client Access Objects 1.5

caoSecurity component

(c) 1998, 1999 Steven P. Goldsmith
All Rights Reserved

The caoSecurity component simplifies AS/400 security by providing a high level
interface to the Client Access Security APIs. Currently only OEM functions are
used.  ANSI and UNICODE functions may be included in a future release.

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 1, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

sgoldsmith@home.com
http://members.home.net/sgoldsmith/
}

unit caoSecurity;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  cwb, cwbsv, cwbsy;

type

  // caoSecurity exception object

  EcaoSecurity = class(Exception);

  // TcaoSecurity component

  TcaoSecurity = class(TComponent)
  private
    { Private declarations }
    FReturnCode: Word;
    FSecHandle: cwbSV_ErrHandle;      // Handle to a security object
    FErrHandle: cwbSY_SecurityHandle; // Handle to a error object
    FsystemName: String;
    FuserID: String;
    Fpassword: String;
    FOnLogon: TNotifyEvent;
    FOnVerifyUserIDPwd: TNotifyEvent;
    FOnChangePwd: TNotifyEvent;
  protected
    { Protected declarations }
    function GetPassword: String; virtual;
    function ResultCodeText: String; virtual;
    function GetErrMsg: String; virtual;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure LogonUser; virtual;
    procedure VerifyUserIDPwd; virtual;
    procedure ChangePwd(NewPassword: String); virtual;
    function GetUserID(const SysName: String): String; virtual;
    procedure Logon; virtual;
    function ConvertDateTime(D: String): TDateTime; virtual;
    function GetDateTimePwdExpires: TDateTime; virtual;
    function GetFailedAttempts: Integer; virtual;
    function GetDateTimeLastSignon: TDateTime; virtual;
    function GetDateTimeCurrentSignon: TDateTime; virtual;
  published
    { Published declarations }
    property ReturnCode: Word read FReturnCode default CWB_OK;
    property systemName: String read FsystemName write FsystemName;
    property userID: String read FuserID write FuserID;
    property password: String read GetPassword write Fpassword;
    property OnLogon: TNotifyEvent read FOnLogon write FOnLogon;
    property OnVerifyUserIDPwd: TNotifyEvent read FOnVerifyUserIDPwd write FOnVerifyUserIDPwd;
    property OnChangePwd: TNotifyEvent read FOnChangePwd write FOnChangePwd;
  end;

implementation

{
Create error and security handles.
}

constructor TcaoSecurity.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FReturnCode := cwbSV_CreateErrHandle(FErrHandle);
  // Raise exception if needed.
  if FReturnCode <> CWB_OK then
    raise EcaoSecurity.Create(GetErrMsg);
  FReturnCode := cwbSY_CreateSecurityObj(FSecHandle);
  // Raise exception if needed.
  if FReturnCode <> CWB_OK then
    raise EcaoSecurity.Create(GetErrMsg)
end;

{
Create error and security handles.
}

destructor TcaoSecurity.Destroy;
begin
  FReturnCode := cwbSY_DeleteSecurityObj(FSecHandle);
  // Raise exception if needed.
  if FReturnCode <> CWB_OK then
    raise EcaoSecurity.Create(GetErrMsg);
  FReturnCode := cwbSV_DeleteErrHandle(FErrHandle);
  // Raise exception if needed.
  if FReturnCode <> CWB_OK then
    raise EcaoSecurity.Create(GetErrMsg);
  inherited Destroy
end;

{
Return formatted result code text.
}

function TcaoSecurity.ResultCodeText: String;
begin
  Result := Format('Result code %d',[FReturnCode])
end;

{
Return error mesage or formatted result code text.
}

function TcaoSecurity.GetErrMsg: String;
var
  S: String;
  ReturnLen: Integer;
  ErrorStr: String;
begin
  SetLength(ErrorStr,255);
  FReturnCode := cwbSV_GetErrText(FErrHandle,PChar(ErrorStr),Length(ErrorStr),ReturnLen);
  S := Copy(ErrorStr,1,ReturnLen-1);
  if S = '' then
    S := ResultCodeText;
  Result := S
end;

{
The API validates the supplied user ID and password without presenting a
sign-on prompt. If they are valid, the information is stored and subsequent
sign-ons and requests for other functions are not prompted for.
}

procedure TcaoSecurity.LogonUser;
begin
  FReturnCode := cwbSY_SetSys(FSecHandle,PChar(FsystemName));
  // Raise exception if needed.
  if FReturnCode <> CWB_OK then
    raise EcaoSecurity.Create(GetErrMsg);
  FReturnCode := cwbSY_LogonUser(
  FSecHandle,PChar(FuserID),PChar(Fpassword),FErrHandle);
  // Raise exception if needed.
  if FReturnCode <> CWB_OK then
    raise EcaoSecurity.Create(GetErrMsg);
  if Assigned(OnLogon) then
    OnLogon(Self)
end;

{
Verifies the user ID and password are valid for the given system.
}

procedure TcaoSecurity.VerifyUserIDPwd;
begin
  FReturnCode := cwbSY_SetSys(FSecHandle,PChar(FsystemName));
  // Raise exception if needed.
  if FReturnCode <> CWB_OK then
    raise EcaoSecurity.Create(GetErrMsg);
  FReturnCode := cwbSY_VerifyUserIDPwd(
  FSecHandle,PChar(FuserID),PChar(Fpassword),FErrHandle);
  // Raise exception if needed.
  if FReturnCode <> CWB_OK then
    raise EcaoSecurity.Create(GetErrMsg);
  if Assigned(OnVerifyUserIDPwd) then
    OnVerifyUserIDPwd(Self)
end;

{
Changes the password on the specified AS/400 system.  The password property is
set to the new password.
}

procedure TcaoSecurity.ChangePwd(NewPassword: String);
begin
  FReturnCode := cwbSY_SetSys(FSecHandle,PChar(FsystemName));
  // Raise exception if needed.
  if FReturnCode <> CWB_OK then
    raise EcaoSecurity.Create(GetErrMsg);
  FReturnCode := cwbSY_ChangePwd(
  FSecHandle,PChar(FuserID),PChar(Fpassword),PChar(NewPassword),FErrHandle);
  // Raise exception if needed.
  if FReturnCode <> CWB_OK then
    raise EcaoSecurity.Create(GetErrMsg);
  // Set new password.
  Fpassword := NewPassword;
  if Assigned(OnChangePwd) then
    OnChangePwd(Self)
end;

{
The API retrieves the currently logged on user ID for the specified system.
}

function TcaoSecurity.GetUserID(const SysName: String): String;
var
  Uid: String;
begin
  SetLength(Uid,10);
  FReturnCode := cwbSY_GetUserID(FSecHandle,PChar(SysName),PChar(Uid));
  // Raise exception if needed.
  if FReturnCode <> CWB_OK then
    raise EcaoSecurity.Create(GetErrMsg);
  Result := Trim(Uid)
end;

{
The API retrieves the currently logged on user ID for the specified system and
stored it in the userID property.
}

procedure TcaoSecurity.Logon;
begin
  FReturnCode := cwbSY_SetSys(FSecHandle,PChar(FsystemName));
  // Raise exception if needed.
  if FReturnCode <> CWB_OK then
    raise EcaoSecurity.Create(GetErrMsg);
  FReturnCode := cwbSY_Logon(FSecHandle);
  // Raise exception if needed.
  if FReturnCode <> CWB_OK then
    raise EcaoSecurity.Create(GetErrMsg);
  if Assigned(OnLogon) then
    OnLogon(Self)
end;

{
Convert AS/400 timestamp to TDateTime.  Timestamp is received as a 8 character
string which will contain the date and time in this format:

   Bytes     Content
   --------  -----------------------------------------------
   1 - 2     Year (Example: 1994 = 0x07CA)
   3         Month (January = 0x01)
   4         Day   (First day = 0x01;31st day = 0x1F)
   5         Hour  (Midnight = 0x01;23rd hour = 0x17)
   6         Minute (On the hour = 0x00; 59th minute = 0x3B)
   7         Second (On the minute = 0x00; 59th second = 0x3B)
   8         One-hundredth of a second (on the second = 0x00;
             maximum = 0x63)

Note: On a given day, the maximum time is 23 hours, 59 minutes, and 59.99
seconds.  Midnight is 0 hours, 0 minutes and 0.0 seconds on the following day.            *)
}

function TcaoSecurity.ConvertDateTime(D: String): TDateTime;
var
  Year: Word;
  cDate: TDateTime;
  cTime: TDateTime;
begin
  Year := Word(Byte(D[1])+Byte(D[2])*256);
  if Year > 0 then
    cDate := EncodeDate(Year,Byte(D[3]),Byte(D[4]))
  else // Set 00/00/0000 date to 01/01/0001
    cDate := EncodeDate(1,1,1);
  cTime := EncodeTime(Byte(D[5]),Byte(D[6]),Byte(D[6]),Byte(D[8]));
  Result := cDate+cTime
end;

{
Retrieves the date and time the password will expire for the given system.  You
must have successfully called cwbSY_verifyUserIDPwd before using this API.                   *)
}

function TcaoSecurity.GetDateTimePwdExpires: TDateTime;
var
  ExpTime: String;
begin
  FReturnCode := cwbSY_SetSys(FSecHandle,PChar(FsystemName));
  // Raise exception if needed.
  if FReturnCode <> CWB_OK then
    raise EcaoSecurity.Create(GetErrMsg);
  SetLength(ExpTime,8);
  FReturnCode := cwbSY_GetDateTimePwdExpires(FSecHandle,PChar(ExpTime));
  // Raise exception if needed.
  if FReturnCode <> CWB_OK then
    raise EcaoSecurity.Create(GetErrMsg);
  Result := ConvertDateTime(ExpTime)
end;

{
Retrieves the number of unsuccessful logon attempts.  You must have successfully
called cwbSY_verifyUserIDPwd before using this API.                                         *)
}

function TcaoSecurity.GetFailedAttempts: Integer;
var
  Attempts: String;
begin
  FReturnCode := cwbSY_SetSys(FSecHandle,PChar(FsystemName));
  // Raise exception if needed.
  if FReturnCode <> CWB_OK then
    raise EcaoSecurity.Create(GetErrMsg);
  SetLength(Attempts,1);
  FReturnCode := cwbSY_GetFailedAttempts(FSecHandle,PChar(Attempts));
  // Raise exception if needed.
  if FReturnCode <> CWB_OK then
    raise EcaoSecurity.Create(GetErrMsg);
  Result := Byte(Attempts[1])
end;

{
Retrieves the date and time of the previous successful signon.  You must have
successfully called cwbSY_verifyUserIDPwd before using this API.
}

function TcaoSecurity.GetDateTimeLastSignon: TDateTime;
var
  LastSignon: String;
begin
  FReturnCode := cwbSY_SetSys(FSecHandle,PChar(FsystemName));
  // Raise exception if needed.
  if FReturnCode <> CWB_OK then
    raise EcaoSecurity.Create(GetErrMsg);
  SetLength(LastSignon,8);
  FReturnCode := cwbSY_GetDateTimeCurrentSignon(FSecHandle,PChar(LastSignon));
  // Raise exception if needed.
  if FReturnCode <> CWB_OK then
    raise EcaoSecurity.Create(GetErrMsg);
  Result := ConvertDateTime(LastSignon)
end;

{
Retrieves the date and time of the current successful signon.  You must have
successfully called cwbSY_verifyUserIDPwd before using this API.                                         *)
}

function TcaoSecurity.GetDateTimeCurrentSignon: TDateTime;
var
  CurrentSignon: String;
begin
  FReturnCode := cwbSY_SetSys(FSecHandle,PChar(FsystemName));
  // Raise exception if needed.
  if FReturnCode <> CWB_OK then
    raise EcaoSecurity.Create(GetErrMsg);
  SetLength(CurrentSignon,8);
  FReturnCode := cwbSY_GetDateTimeCurrentSignon(FSecHandle,PChar(CurrentSignon));
  // Raise exception if needed.
  if FReturnCode <> CWB_OK then
    raise EcaoSecurity.Create(GetErrMsg);
  Result := ConvertDateTime(CurrentSignon)
end;

{
Get password.  If in design mode then get * out password.
}

function TcaoSecurity.GetPassword: String;
begin
  Result := Fpassword;
  if ComponentState = [csDesigning] then
    FillChar(Result[1],Length(Result),'*')
end;


end.
